# Generated by Django 1.10.7 on 2017-07-07 09:23
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cspreports', '0002_auto_20141011_1800'),
    ]

    operations = [
        migrations.AddField(
            model_name='cspreport',
            name='blocked_uri',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='column_number',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='disposition',
            field=models.CharField(blank=True, choices=[('enforce', 'enforce'), ('report', 'report')], max_length=10, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='document_uri',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='effective_directive',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='is_valid',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='line_number',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='original_policy',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='referrer',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='source_file',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='status_code',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cspreport',
            name='violated_directive',
            field=models.TextField(blank=True, null=True),
        ),
    ]
