# Generated by Django 1.10.8 on 2017-11-29 08:09
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cspreports', '0003_report_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='cspreport',
            name='user_agent',
            field=models.TextField(blank=True),
        ),
    ]
